@@ -19,24 +19,38 @@
 package org.apache.felix.bundleplugin;
 
 
+import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.lang.reflect.Method;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collection;
+import java.util.Collections;
+import java.util.Enumeration;
+import java.util.HashSet;
+import java.util.Iterator;
+import java.util.LinkedHashMap;
+import java.util.LinkedHashSet;
+import java.util.List;
+import java.util.Map;
+import java.util.Properties;
+import java.util.Set;
 import java.util.jar.Attributes;
 import java.util.jar.Manifest;
 
-import aQute.lib.osgi.*;
 import org.apache.maven.archiver.ManifestSection;
 import org.apache.maven.archiver.MavenArchiveConfiguration;
 import org.apache.maven.archiver.MavenArchiver;
 import org.apache.maven.artifact.Artifact;
 import org.apache.maven.artifact.handler.manager.ArtifactHandlerManager;
+import org.apache.maven.execution.MavenSession;
 import org.apache.maven.model.License;
 import org.apache.maven.model.Model;
+import org.apache.maven.model.Resource;
 import org.apache.maven.plugin.AbstractMojo;
 import org.apache.maven.plugin.MojoExecutionException;
 import org.apache.maven.plugin.MojoFailureException;
@@ -48,9 +62,16 @@ import org.apache.maven.shared.osgi.Maven2OsgiConverter;
 import org.codehaus.plexus.archiver.UnArchiver;
 import org.codehaus.plexus.archiver.manager.ArchiverManager;
 import org.codehaus.plexus.util.DirectoryScanner;
-import org.codehaus.plexus.util.StringInputStream;
+import org.codehaus.plexus.util.FileUtils;
 import org.codehaus.plexus.util.StringUtils;
 
+import aQute.lib.osgi.Analyzer;
+import aQute.lib.osgi.Builder;
+import aQute.lib.osgi.Constants;
+import aQute.lib.osgi.EmbeddedResource;
+import aQute.lib.osgi.FileResource;
+import aQute.lib.osgi.Jar;
+import aQute.lib.osgi.Processor;
 import aQute.lib.spring.SpringXMLType;
 
 
@@ -61,6 +82,7 @@ import aQute.lib.spring.SpringXMLType;
  * @phase package
  * @requiresDependencyResolution test
  * @description build an OSGi bundle jar
+ * @threadSafe
  */
 public class BundlePlugin extends AbstractMojo
 {
@@ -71,6 +93,20 @@ public class BundlePlugin extends AbstractMojo
      */
     protected File manifestLocation;
 
+    /**
+     * File where the BND instructions will be dumped
+     *
+     * @parameter expression="${dumpInstructions}"
+     */
+    protected File dumpInstructions;
+
+    /**
+     * File where the BND class-path will be dumped
+     *
+     * @parameter expression="${dumpClasspath}"
+     */
+    protected File dumpClasspath;
+
     /**
      * When true, unpack the bundle contents to the outputDirectory
      *
@@ -124,14 +160,6 @@ public class BundlePlugin extends AbstractMojo
      */
     private File outputDirectory;
 
-    /**
-     * The directory for the pom
-     *
-     * @parameter expression="${basedir}"
-     * @required
-     */
-    private File baseDir;
-
     /**
      * The directory for the generated JAR.
      *
@@ -168,6 +196,13 @@ public class BundlePlugin extends AbstractMojo
      */
     private MavenArchiveConfiguration archive; // accessed indirectly in JarPluginConfiguration
 
+    /**
+     * @parameter default-value="${session}"
+     * @required
+     * @readonly
+     */
+    private MavenSession m_mavenSession;
+
     private static final String MAVEN_SYMBOLICNAME = "maven-symbolicname";
     private static final String MAVEN_RESOURCES = "{maven-resources}";
     private static final String LOCAL_PACKAGES = "{local-packages}";
@@ -177,6 +212,8 @@ public class BundlePlugin extends AbstractMojo
     private static final String[] DEFAULT_INCLUDES =
         { "**/**" };
 
+    private static final String NL = System.getProperty( "line.separator" );
+
 
     protected Maven2OsgiConverter getMaven2OsgiConverter()
     {
@@ -254,36 +291,59 @@ public class BundlePlugin extends AbstractMojo
                 value = value.replaceAll( "\\p{Blank}*[\r\n]\\p{Blank}*", "" );
             }
 
+            if ( Analyzer.WAB.equals( key ) && value.length() == 0 )
+            {
+                // provide useful default
+                value = "src/main/webapp/";
+            }
+
             transformedInstructions.put( key, value );
         }
         return transformedInstructions;
     }
 
 
+    protected boolean reportErrors( String prefix, Analyzer analyzer )
+    {
+        List errors = analyzer.getErrors();
+        List warnings = analyzer.getWarnings();
+
+        for ( Iterator w = warnings.iterator(); w.hasNext(); )
+        {
+            String msg = ( String ) w.next();
+            getLog().warn( prefix + " : " + msg );
+        }
+
+        boolean hasErrors = false;
+        String fileNotFound = "Input file does not exist: ";
+        for ( Iterator e = errors.iterator(); e.hasNext(); )
+        {
+            String msg = ( String ) e.next();
+            if ( msg.startsWith( fileNotFound ) && msg.endsWith( "~" ) )
+            {
+                // treat as warning; this error happens when you have duplicate entries in Include-Resource
+                String duplicate = Processor.removeDuplicateMarker( msg.substring( fileNotFound.length() ) );
+                getLog().warn( prefix + " : Duplicate path '" + duplicate + "' in Include-Resource" );
+            }
+            else
+            {
+                getLog().error( prefix + " : " + msg );
+                hasErrors = true;
+            }
+        }
+        return hasErrors;
+    }
+
+
     protected void execute( MavenProject currentProject, Map originalInstructions, Properties properties,
         Jar[] classpath ) throws MojoExecutionException
     {
         try
         {
             File jarFile = new File( getBuildDirectory(), getBundleName( currentProject ) );
-
             Builder builder = buildOSGiBundle( currentProject, originalInstructions, properties, classpath );
-
-            List errors = builder.getErrors();
-            List warnings = builder.getWarnings();
-
-            for ( Iterator w = warnings.iterator(); w.hasNext(); )
-            {
-                String msg = ( String ) w.next();
-                getLog().warn( "Warning building bundle " + currentProject.getArtifact() + " : " + msg );
-            }
-            for ( Iterator e = errors.iterator(); e.hasNext(); )
-            {
-                String msg = ( String ) e.next();
-                getLog().error( "Error building bundle " + currentProject.getArtifact() + " : " + msg );
-            }
-
-            if ( errors.size() > 0 )
+            boolean hasErrors = reportErrors( "Bundle " + currentProject.getArtifact(), builder );
+            if ( hasErrors )
             {
                 String failok = builder.getProperty( "-failok" );
                 if ( null == failok || "false".equalsIgnoreCase( failok ) )
@@ -300,8 +360,11 @@ public class BundlePlugin extends AbstractMojo
 
             Artifact mainArtifact = currentProject.getArtifact();
 
-            // workaround for MNG-1682: force maven to install artifact using the "jar" handler
-            mainArtifact.setArtifactHandler( m_artifactHandlerManager.getArtifactHandler( "jar" ) );
+            if ( "bundle".equals( mainArtifact.getType() ) )
+            {
+                // workaround for MNG-1682: force maven to install artifact using the "jar" handler
+                mainArtifact.setArtifactHandler( m_artifactHandlerManager.getArtifactHandler( "jar" ) );
+            }
 
             if ( null == classifier || classifier.trim().length() == 0 )
             {
@@ -348,91 +411,148 @@ public class BundlePlugin extends AbstractMojo
     }
 
 
-    protected Builder buildOSGiBundle( MavenProject currentProject, Map originalInstructions, Properties properties,
+    protected Builder getOSGiBuilder( MavenProject currentProject, Map originalInstructions, Properties properties,
         Jar[] classpath ) throws Exception
     {
         properties.putAll( getDefaultProperties( currentProject ) );
         properties.putAll( transformDirectives( originalInstructions ) );
 
         Builder builder = new Builder();
-        builder.setBase( currentProject.getBasedir() );
+        builder.setBase( getBase( currentProject ) );
         builder.setProperties( properties );
-        builder.setClasspath( classpath );
-
-        // update BND instructions to add included Maven resources
-        includeMavenResources( currentProject, builder, getLog() );
-
-        // calculate default export/private settings based on sources
-        addLocalPackages( outputDirectory, builder );
-
-        // update BND instructions to embed selected Maven dependencies
-        Collection embeddableArtifacts = getEmbeddableArtifacts( currentProject, builder );
-        new DependencyEmbedder( getLog(), embeddableArtifacts ).processHeaders( builder );
-
-        dumpInstructions( "BND Instructions:", builder.getProperties(), getLog() );
-        dumpClasspath( "BND Classpath:", builder.getClasspath(), getLog() );
-
-        builder.build();
-        Jar jar = builder.getJar();
-
-        dumpManifest( "BND Manifest:", jar.getManifest(), getLog() );
-
-        String[] removeHeaders = builder.getProperty( Constants.REMOVEHEADERS, "" ).split( "," );
-
-        mergeMavenManifest( currentProject, jar, removeHeaders, getLog() );
-        builder.setJar( jar );
-
-        dumpManifest( "Final Manifest:", jar.getManifest(), getLog() );
+        if ( classpath != null )
+        {
+            builder.setClasspath( classpath );
+        }
 
         return builder;
     }
 
 
-    protected static void dumpInstructions( String title, Properties properties, Log log )
+    protected void addMavenInstructions( MavenProject currentProject, Builder builder ) throws Exception
     {
-        if ( log.isDebugEnabled() )
+        if ( currentProject.getBasedir() != null )
         {
-            log.debug( title );
-            log.debug( "------------------------------------------------------------------------" );
-            for ( Enumeration e = properties.propertyNames(); e.hasMoreElements(); )
+            // update BND instructions to add included Maven resources
+            includeMavenResources( currentProject, builder, getLog() );
+
+            // calculate default export/private settings based on sources
+            addLocalPackages( outputDirectory, builder );
+        }
+
+        // update BND instructions to embed selected Maven dependencies
+        Collection embeddableArtifacts = getEmbeddableArtifacts( currentProject, builder );
+        new DependencyEmbedder( getLog(), embeddableArtifacts ).processHeaders( builder );
+
+        if ( dumpInstructions != null || getLog().isDebugEnabled() )
+        {
+            StringBuilder buf = new StringBuilder();
+            getLog().debug( "BND Instructions:" + NL + dumpInstructions( builder.getProperties(), buf ) );
+            if ( dumpInstructions != null )
             {
-                String key = ( String ) e.nextElement();
-                log.debug( key + ": " + properties.getProperty( key ) );
+                getLog().info( "Writing BND instructions to " + dumpInstructions );
+                dumpInstructions.getParentFile().mkdirs();
+                FileUtils.fileWrite( dumpInstructions, "# BND instructions" + NL + buf );
+            }
+        }
+
+        if ( dumpClasspath != null || getLog().isDebugEnabled() )
+        {
+            StringBuilder buf = new StringBuilder();
+            getLog().debug( "BND Classpath:" + NL + dumpClasspath( builder.getClasspath(), buf ) );
+            if ( dumpClasspath != null )
+            {
+                getLog().info( "Writing BND classpath to " + dumpClasspath );
+                dumpClasspath.getParentFile().mkdirs();
+                FileUtils.fileWrite( dumpClasspath, "# BND classpath" + NL + buf );
             }
-            log.debug( "------------------------------------------------------------------------" );
         }
     }
 
 
-    protected static void dumpClasspath( String title, List classpath, Log log )
+    protected Builder buildOSGiBundle( MavenProject currentProject, Map originalInstructions, Properties properties,
+        Jar[] classpath ) throws Exception
     {
-        if ( log.isDebugEnabled() )
+        Builder builder = getOSGiBuilder( currentProject, originalInstructions, properties, classpath );
+
+        addMavenInstructions( currentProject, builder );
+
+        builder.build();
+
+        mergeMavenManifest( currentProject, builder );
+
+        return builder;
+    }
+
+
+    protected static StringBuilder dumpInstructions( Properties properties, StringBuilder buf )
+    {
+        try
         {
-            log.debug( title );
-            log.debug( "------------------------------------------------------------------------" );
+            buf.append( "#-----------------------------------------------------------------------" + NL );
+            Properties stringProperties = new Properties();
+            for ( Enumeration e = properties.propertyNames(); e.hasMoreElements(); )
+            {
+                // we can only store String properties
+                String key = ( String ) e.nextElement();
+                String value = properties.getProperty( key );
+                if ( value != null )
+                {
+                    stringProperties.setProperty( key, value );
+                }
+            }
+            ByteArrayOutputStream out = new ByteArrayOutputStream();
+            stringProperties.store( out, null ); // properties encoding is 8859_1
+            buf.append( out.toString( "8859_1" ) );
+            buf.append( "#-----------------------------------------------------------------------" + NL );
+        }
+        catch ( Throwable e )
+        {
+            // ignore...
+        }
+        return buf;
+    }
+
+
+    protected static StringBuilder dumpClasspath( List classpath, StringBuilder buf )
+    {
+        try
+        {
+            buf.append( "#-----------------------------------------------------------------------" + NL );
+            buf.append( "-classpath:\\" + NL );
             for ( Iterator i = classpath.iterator(); i.hasNext(); )
             {
                 File path = ( ( Jar ) i.next() ).getSource();
-                log.debug( null == path ? "null" : path.toString() );
+                if ( path != null )
+                {
+                    buf.append( ' ' + path.toString() + ( i.hasNext() ? ",\\" : "" ) + NL );
+                }
             }
-            log.debug( "------------------------------------------------------------------------" );
+            buf.append( "#-----------------------------------------------------------------------" + NL );
         }
+        catch ( Throwable e )
+        {
+            // ignore...
+        }
+        return buf;
     }
 
 
-    protected static void dumpManifest( String title, Manifest manifest, Log log )
+    protected static StringBuilder dumpManifest( Manifest manifest, StringBuilder buf )
     {
-        if ( log.isDebugEnabled() )
+        try
         {
-            log.debug( title );
-            log.debug( "------------------------------------------------------------------------" );
-            for ( Iterator i = manifest.getMainAttributes().entrySet().iterator(); i.hasNext(); )
-            {
-                Map.Entry entry = ( Map.Entry ) i.next();
-                log.debug( entry.getKey() + ": " + entry.getValue() );
-            }
-            log.debug( "------------------------------------------------------------------------" );
+            buf.append( "#-----------------------------------------------------------------------" + NL );
+            ByteArrayOutputStream out = new ByteArrayOutputStream();
+            manifest.write( out ); // manifest encoding is UTF8
+            buf.append( out.toString( "UTF8" ) );
+            buf.append( "#-----------------------------------------------------------------------" + NL );
         }
+        catch ( Throwable e )
+        {
+            // ignore...
+        }
+        return buf;
     }
 
 
@@ -479,10 +599,16 @@ public class BundlePlugin extends AbstractMojo
     }
 
 
-    protected void mergeMavenManifest( MavenProject currentProject, Jar jar, String[] removeHeaders, Log log )
-        throws IOException
+    protected void mergeMavenManifest( MavenProject currentProject, Builder builder ) throws Exception
     {
-        boolean addMavenDescriptor = true;
+        Jar jar = builder.getJar();
+
+        if ( getLog().isDebugEnabled() )
+        {
+            getLog().debug( "BND Manifest:" + NL + dumpManifest( jar.getManifest(), new StringBuilder() ) );
+        }
+
+        boolean addMavenDescriptor = currentProject.getBasedir() != null;
 
         try
         {
@@ -491,7 +617,7 @@ public class BundlePlugin extends AbstractMojo
              */
             MavenArchiveConfiguration archiveConfig = JarPluginConfiguration.getArchiveConfiguration( currentProject );
             String mavenManifestText = new MavenArchiver().getManifest( currentProject, archiveConfig ).toString();
-            addMavenDescriptor = archiveConfig.isAddMavenDescriptor();
+            addMavenDescriptor = addMavenDescriptor && archiveConfig.isAddMavenDescriptor();
 
             Manifest mavenManifest = new Manifest();
 
@@ -504,8 +630,8 @@ public class BundlePlugin extends AbstractMojo
                 mis.close();
             }
 
-            // Then apply the customized entries from the jar plugin
-            mavenManifest.read( new StringInputStream( mavenManifestText ) );
+            // Then apply customized entries from the jar plugin; note: manifest encoding is UTF8
+            mavenManifest.read( new ByteArrayInputStream( mavenManifestText.getBytes( "UTF8" ) ) );
 
             if ( !archiveConfig.isManifestSectionsEmpty() )
             {
@@ -535,6 +661,8 @@ public class BundlePlugin extends AbstractMojo
             Attributes mainMavenAttributes = mavenManifest.getMainAttributes();
             mainMavenAttributes.putValue( "Created-By", "Apache Maven Bundle Plugin" );
 
+            String[] removeHeaders = builder.getProperty( Constants.REMOVEHEADERS, "" ).split( "," );
+
             // apply -removeheaders to the custom manifest
             for ( int i = 0; i < removeHeaders.length; i++ )
             {
@@ -556,33 +684,43 @@ public class BundlePlugin extends AbstractMojo
 
             // adjust the import package attributes so that optional dependencies use
             // optional resolution.
-            String importPackages = bundleManifest.getMainAttributes().getValue("Import-Package");
-            if( importPackages!=null ) {
-                Set optionalPackages = getOptionalPackages(currentProject);
+            String importPackages = bundleManifest.getMainAttributes().getValue( "Import-Package" );
+            if ( importPackages != null )
+            {
+                Set optionalPackages = getOptionalPackages( currentProject );
 
-                Map<String, Map<String, String>> values = new Analyzer().parseHeader(importPackages);
-                for (Map.Entry<String, Map<String, String>> entry: values.entrySet()) {
+                Map<String, Map<String, String>> values = new Analyzer().parseHeader( importPackages );
+                for ( Map.Entry<String, Map<String, String>> entry : values.entrySet() )
+                {
                     String pkg = entry.getKey();
                     Map<String, String> options = entry.getValue();
-                    if( !options.containsKey("resolution:") && optionalPackages.contains(pkg) ) {
-                        options.put("resolution:", "optional");
+                    if ( !options.containsKey( "resolution:" ) && optionalPackages.contains( pkg ) )
+                    {
+                        options.put( "resolution:", "optional" );
                     }
                 }
-                String result = Processor.printClauses(values, "resolution:");
-                bundleManifest.getMainAttributes().putValue("Import-Package", result);
+                String result = Processor.printClauses( values, "resolution:" );
+                bundleManifest.getMainAttributes().putValue( "Import-Package", result );
             }
 
             jar.setManifest( bundleManifest );
         }
         catch ( Exception e )
         {
-            log.warn( "Unable to merge Maven manifest: " + e.getLocalizedMessage() );
+            getLog().warn( "Unable to merge Maven manifest: " + e.getLocalizedMessage() );
         }
 
         if ( addMavenDescriptor )
         {
             doMavenMetadata( currentProject, jar );
         }
+
+        if ( getLog().isDebugEnabled() )
+        {
+            getLog().debug( "Final Manifest:" + NL + dumpManifest( jar.getManifest(), new StringBuilder() ) );
+        }
+
+        builder.setJar( jar );
     }
 
 
@@ -597,7 +735,7 @@ public class BundlePlugin extends AbstractMojo
             {
                 if ( !Artifact.SCOPE_TEST.equals( artifact.getScope() ) )
                 {
-                    inscope.add(artifact);
+                    inscope.add( artifact );
                 }
             }
         }
@@ -606,13 +744,15 @@ public class BundlePlugin extends AbstractMojo
         for ( Iterator it = inscope.iterator(); it.hasNext(); )
         {
             Artifact artifact = ( Artifact ) it.next();
-            if( artifact.isOptional() ) {
+            if ( artifact.isOptional() )
+            {
                 String id = artifact.toString();
-                if( artifact.getScope()!=null ) {
+                if ( artifact.getScope() != null )
+                {
                     // strip the scope...
-                    id = id.replaceFirst(":[^:]*$", "");
+                    id = id.replaceFirst( ":[^:]*$", "" );
                 }
-                optionalArtifactIds.add(id);
+                optionalArtifactIds.add( id );
             }
 
         }
@@ -629,35 +769,43 @@ public class BundlePlugin extends AbstractMojo
             }
 
             Jar jar = new Jar( artifact.getArtifactId(), file );
-            if( isTransitivelyOptional(optionalArtifactIds, artifact) ) {
-                optional.addAll(jar.getPackages());
-            } else {
-                required.addAll(jar.getPackages());
+            if ( isTransitivelyOptional( optionalArtifactIds, artifact ) )
+            {
+                optional.addAll( jar.getPackages() );
+            }
+            else
+            {
+                required.addAll( jar.getPackages() );
             }
             jar.close();
         }
 
-        optional.removeAll(required);
+        optional.removeAll( required );
         return optional;
     }
 
+
     /**
      * Check to see if any dependency along the dependency trail of
      * the artifact is optional.
      *
      * @param artifact
      */
-    protected boolean isTransitivelyOptional(HashSet optionalArtifactIds, Artifact artifact) {
+    protected boolean isTransitivelyOptional( HashSet optionalArtifactIds, Artifact artifact )
+    {
         List trail = artifact.getDependencyTrail();
-        for (Iterator iterator = trail.iterator(); iterator.hasNext();) {
-            String next = (String) iterator.next();
-            if( optionalArtifactIds.contains(next) ) {
+        for ( Iterator iterator = trail.iterator(); iterator.hasNext(); )
+        {
+            String next = ( String ) iterator.next();
+            if ( optionalArtifactIds.contains( next ) )
+            {
                 return true;
             }
         }
         return false;
     }
 
+
     private void unpackBundle( File jarFile )
     {
         File outputDir = getOutputDirectory();
@@ -772,7 +920,7 @@ public class BundlePlugin extends AbstractMojo
     private void doMavenMetadata( MavenProject currentProject, Jar jar ) throws IOException
     {
         String path = "META-INF/maven/" + currentProject.getGroupId() + "/" + currentProject.getArtifactId();
-        File pomFile = new File( baseDir, "pom.xml" );
+        File pomFile = new File( currentProject.getBasedir(), "pom.xml" );
         jar.putResource( path + "/pom.xml", new FileResource( pomFile ) );
 
         Properties p = new Properties();
@@ -781,9 +929,7 @@ public class BundlePlugin extends AbstractMojo
         p.put( "artifactId", currentProject.getArtifactId() );
         ByteArrayOutputStream out = new ByteArrayOutputStream();
         p.store( out, "Generated by org.apache.felix.bundleplugin" );
-        jar
-            .putResource( path + "/pom.properties",
-                new EmbeddedResource( out.toByteArray(), System.currentTimeMillis() ) );
+        jar.putResource( path + "/pom.properties", new EmbeddedResource( out.toByteArray(), System.currentTimeMillis() ) );
     }
 
 
@@ -883,12 +1029,25 @@ public class BundlePlugin extends AbstractMojo
      */
     protected String getBundleName( MavenProject currentProject )
     {
+        String extension;
+        try
+        {
+            extension = currentProject.getArtifact().getArtifactHandler().getExtension();
+        }
+        catch ( Throwable e )
+        {
+            extension = currentProject.getArtifact().getType();
+        }
+        if ( StringUtils.isEmpty( extension ) || "bundle".equals( extension ) || "pom".equals( extension ) )
+        {
+            extension = "jar"; // just in case maven gets confused
+        }
         String finalName = currentProject.getBuild().getFinalName();
         if ( null != classifier && classifier.trim().length() > 0 )
         {
-            return finalName + '-' + classifier + ".jar";
+            return finalName + '-' + classifier + '.' + extension;
         }
-        return finalName + ".jar";
+        return finalName + '.' + extension;
     }
 
 
@@ -922,7 +1081,7 @@ public class BundlePlugin extends AbstractMojo
         properties.put( MAVEN_SYMBOLICNAME, bsn );
         properties.put( Analyzer.BUNDLE_SYMBOLICNAME, bsn );
         properties.put( Analyzer.IMPORT_PACKAGE, "*" );
-        properties.put( Analyzer.BUNDLE_VERSION, getMaven2OsgiConverter().getVersion(currentProject.getVersion()) );
+        properties.put( Analyzer.BUNDLE_VERSION, getMaven2OsgiConverter().getVersion( currentProject.getVersion() ) );
 
         // remove the extraneous Include-Resource and Private-Package entries from generated manifest
         properties.put( Constants.REMOVEHEADERS, Analyzer.INCLUDE_RESOURCE + ',' + Analyzer.PRIVATE_PACKAGE );
@@ -937,10 +1096,13 @@ public class BundlePlugin extends AbstractMojo
 
         if ( currentProject.getOrganization() != null )
         {
-            String organizationName = currentProject.getOrganization().getName();
-            header( properties, Analyzer.BUNDLE_VENDOR, organizationName );
-            properties.put( "project.organization.name", organizationName );
-            properties.put( "pom.organization.name", organizationName );
+            if ( currentProject.getOrganization().getName() != null )
+            {
+                String organizationName = currentProject.getOrganization().getName();
+                header( properties, Analyzer.BUNDLE_VENDOR, organizationName );
+                properties.put( "project.organization.name", organizationName );
+                properties.put( "pom.organization.name", organizationName );
+            }
             if ( currentProject.getOrganization().getUrl() != null )
             {
                 String organizationUrl = currentProject.getOrganization().getUrl();
@@ -952,27 +1114,31 @@ public class BundlePlugin extends AbstractMojo
 
         properties.putAll( currentProject.getProperties() );
         properties.putAll( currentProject.getModel().getProperties() );
+        if ( m_mavenSession != null )
+        {
+            properties.putAll( m_mavenSession.getExecutionProperties() );
+        }
+
         properties.putAll( getProperties( currentProject.getModel(), "project.build." ) );
         properties.putAll( getProperties( currentProject.getModel(), "pom." ) );
         properties.putAll( getProperties( currentProject.getModel(), "project." ) );
-        properties.put( "project.baseDir", baseDir );
+
+        properties.put( "project.baseDir", getBase( currentProject ) );
         properties.put( "project.build.directory", getBuildDirectory() );
         properties.put( "project.build.outputdirectory", getOutputDirectory() );
 
         properties.put( "classifier", classifier == null ? "" : classifier );
 
         // Add default plugins
-        header( properties, Analyzer.PLUGIN,
-                    BlueprintPlugin.class.getName() + ","
-                        + SpringXMLType.class.getName());
+        header( properties, Analyzer.PLUGIN, BlueprintPlugin.class.getName() + "," + SpringXMLType.class.getName() );
 
         return properties;
     }
 
 
-    protected void setBasedir( File _basedir )
+    protected static File getBase( MavenProject currentProject )
     {
-        baseDir = _basedir;
+        return currentProject.getBasedir() != null ? currentProject.getBasedir() : new File( "" );
     }
 
 
@@ -1012,8 +1178,8 @@ public class BundlePlugin extends AbstractMojo
 
         StringBuffer exportedPkgs = new StringBuffer();
         StringBuffer privatePkgs = new StringBuffer();
-        
-        boolean noprivatePackages = "!*".equals(analyzer.getProperty( Analyzer.PRIVATE_PACKAGE ) );
+
+        boolean noprivatePackages = "!*".equals( analyzer.getProperty( Analyzer.PRIVATE_PACKAGE ) );
 
         for ( Iterator i = packages.iterator(); i.hasNext(); )
         {
@@ -1025,7 +1191,7 @@ public class BundlePlugin extends AbstractMojo
             // we can't export the default package (".") and we shouldn't export internal packages 
             if ( noprivatePackages || !( ".".equals( pkg ) || pkg.contains( ".internal" ) || pkg.contains( ".impl" ) ) )
             {
-                if( exportedPkgs.length() > 0 )
+                if ( exportedPkgs.length() > 0 )
                 {
                     exportedPkgs.append( ';' );
                 }
@@ -1049,7 +1215,7 @@ public class BundlePlugin extends AbstractMojo
         else
         {
             String exported = analyzer.getProperty( Analyzer.EXPORT_PACKAGE );
-            if( exported.indexOf( LOCAL_PACKAGES ) >= 0 )
+            if ( exported.indexOf( LOCAL_PACKAGES ) >= 0 )
             {
                 String newExported = StringUtils.replace( exported, LOCAL_PACKAGES, exportedPkgs.toString() );
                 analyzer.setProperty( Analyzer.EXPORT_PACKAGE, newExported );
@@ -1072,14 +1238,36 @@ public class BundlePlugin extends AbstractMojo
     }
 
 
-    private static String getMavenResourcePaths( MavenProject project )
+    private static List getMavenResources( MavenProject currentProject )
     {
-        final String basePath = project.getBasedir().getAbsolutePath();
+        List resources = new ArrayList( currentProject.getResources() );
+
+        if ( currentProject.getCompileSourceRoots() != null )
+        {
+            // also scan for any "packageinfo" files lurking in the source folders
+            List packageInfoIncludes = Collections.singletonList( "**/packageinfo" );
+            for ( Iterator i = currentProject.getCompileSourceRoots().iterator(); i.hasNext(); )
+            {
+                String sourceRoot = ( String ) i.next();
+                Resource packageInfoResource = new Resource();
+                packageInfoResource.setDirectory( sourceRoot );
+                packageInfoResource.setIncludes( packageInfoIncludes );
+                resources.add( packageInfoResource );
+            }
+        }
+
+        return resources;
+    }
+
+
+    protected static String getMavenResourcePaths( MavenProject currentProject )
+    {
+        final String basePath = currentProject.getBasedir().getAbsolutePath();
 
         Set pathSet = new LinkedHashSet();
-        for ( Iterator i = project.getResources().iterator(); i.hasNext(); )
+        for ( Iterator i = getMavenResources( currentProject ).iterator(); i.hasNext(); )
         {
-            org.apache.maven.model.Resource resource = ( org.apache.maven.model.Resource ) i.next();
+            Resource resource = ( Resource ) i.next();
 
             final String sourcePath = resource.getDirectory();
             final String targetPath = resource.getTargetPath();
@@ -1089,7 +1277,7 @@ public class BundlePlugin extends AbstractMojo
             {
                 DirectoryScanner scanner = new DirectoryScanner();
 
-                scanner.setBasedir( resource.getDirectory() );
+                scanner.setBasedir( sourcePath );
                 if ( resource.getIncludes() != null && !resource.getIncludes().isEmpty() )
                 {
                     scanner.setIncludes( ( String[] ) resource.getIncludes().toArray( EMPTY_STRING_ARRAY ) );
@@ -1154,7 +1342,7 @@ public class BundlePlugin extends AbstractMojo
         }
 
         StringBuffer resourcePaths = new StringBuffer();
-        for ( Iterator i = pathSet.iterator() ; i.hasNext(); )
+        for ( Iterator i = pathSet.iterator(); i.hasNext(); )
         {
             resourcePaths.append( i.next() );
             if ( i.hasNext() )
@@ -1167,7 +1355,7 @@ public class BundlePlugin extends AbstractMojo
     }
 
 
-    protected Collection getEmbeddableArtifacts( MavenProject project, Analyzer analyzer )
+    protected Collection getEmbeddableArtifacts( MavenProject currentProject, Analyzer analyzer )
         throws MojoExecutionException
     {
         final Collection artifacts;
@@ -1176,12 +1364,12 @@ public class BundlePlugin extends AbstractMojo
         if ( Boolean.valueOf( embedTransitive ).booleanValue() )
         {
             // includes transitive dependencies
-            artifacts = project.getArtifacts();
+            artifacts = currentProject.getArtifacts();
         }
         else
         {
             // only includes direct dependencies
-            artifacts = project.getDependencyArtifacts();
+            artifacts = currentProject.getDependencyArtifacts();
         }
 
         return getSelectedDependencies( artifacts );
